import { ElementRef, Injectable } from '@angular/core';
import { UntypedFormGroup } from '@angular/forms';
import * as _ from 'lodash';
import { Owner } from '../models/owner.model';
import { Structure } from '../models/structure.model';
import { User } from '../models/user.model';
import { OtherServices } from '../shared/enum/otherServices.enum';
import { AccessModality } from '../structure-list/enum/access-modality.enum';
import { Equipment } from '../structure-list/enum/equipment.enum';
import { Category } from '../structure-list/models/category.model';
import { Filter } from '../structure-list/models/filter.model';
import { Module } from '../structure-list/models/module.model';
import { PersonalOffer } from './../models/personalOffer.model';

export type CanExitResolver = (value: boolean) => void;

@Injectable({
  providedIn: 'root',
})
export class Utils {
  public modifyPhoneValue(phoneNumber: string): string {
    // Take length of phone number without spaces.
    const phoneNoSpace = phoneNumber.replace(/\s/g, '');
    // Check to refresh every 2 number.
    if (phoneNoSpace.length % 2 === 0) {
      // Add space every 2 number
      return phoneNoSpace.replace(/(?!^)(?=(?:\d{2})+$)/g, ' '); // NOSONAR
    }
    return phoneNumber;
  }

  public getJobEmployer(userProfile: User | Owner): string {
    const jobName = userProfile.job?.name;
    const employerName = userProfile.employer?.name;
    if (!jobName && !employerName) return '';
    if (!jobName && employerName) return employerName;
    if (jobName && !employerName) return jobName;
    if (jobName && employerName) return `${jobName}, ${employerName}`;
  }

  public getAccessLabel(accessModality: AccessModality): string {
    switch (accessModality) {
      case AccessModality.free:
        return 'Accès libre';
      case AccessModality.meeting:
        return 'Sur rendez-vous';
      case AccessModality.meetingOnly:
        return 'Uniquement sur RDV';
      case AccessModality.numeric:
        return 'Téléphone / Visio';
      default:
        return null;
    }
  }

  public hasSocialNetwork(structure: Structure): boolean {
    return (
      (structure.facebook !== null && structure.facebook !== '') ||
      (structure.instagram !== null && structure.instagram !== '') ||
      (structure.linkedin !== null && structure.linkedin !== '') ||
      (structure.twitter !== null && structure.twitter !== '')
    );
  }

  public hasWebsite(structure: Structure): boolean {
    return structure.website !== null && structure.website !== '';
  }

  public hasWifi(selfServiceMaterial: string[]): boolean {
    return selfServiceMaterial.includes('wifiEnAccesLibre');
  }

  /**
   * Verify that a structure has equipments and checks that their value is > 0
   */
  public hasEquipments(nbComputers: number, nbPrinters: number, nbScanners: number): boolean {
    return nbComputers + nbPrinters + nbScanners > 0;
  }

  public getEquipmentsLabelAndValue(equipment: Equipment, number: number): string {
    switch (equipment) {
      case Equipment.wifi:
        return 'Wifi en accès libre';
      case Equipment.printer:
        return `Imprimante${number > 1 ? 's' : ''} : ${number}`;
      case Equipment.computer:
        return `Ordinateur${number > 1 ? 's' : ''} : ${number}`;
      case Equipment.scanner:
        return `Scanner${number > 1 ? 's' : ''} : ${number}`;
      default:
        return null;
    }
  }

  public getOtherServicesLabel(otherService: OtherServices): string {
    return OtherServices[otherService];
  }

  public convertFiltersToModule(filters: Filter[]): Module[] {
    const modules: Module[] = [];
    // Add checked box filter + special handling for pmrAccess that is not a module like the rest of accessModality items
    filters.forEach((filter) => {
      if (filter.name !== 'query') {
        if (filter.name === 'pmrAccess') {
          modules.push(new Module('True', 'accessModality', 'Accessible PMR'));
        } else {
          modules.push(new Module(filter.value, filter.name, filter.text));
        }
      }
    });
    return modules;
  }

  /**
   * Map categories ids to their real names
   */
  public setServiceCategories(categories: Category[], structure: Structure | PersonalOffer): Structure | PersonalOffer {
    categories.forEach((category) => {
      const structureModuleIds = structure.categories[category.id];
      if (structureModuleIds) {
        const moduleNames = category.modules
          .map((module) => {
            if (structureModuleIds.includes(module.id)) {
              return module;
            }
          })
          .filter((value) => value !== undefined);
        structure.categoriesDisplay = {
          ...structure.categoriesDisplay,
          [category.id]: moduleNames,
        };
      }
    });
    return structure;
  }

  /**
   * Map categories ids to their real names for structure and all its personal offers
   */
  public setServiceCategoriesWithPersonalOffers(categories: Category[], structure: Structure): void {
    // Set category names in this.structure.categoriesDisplay
    this.setServiceCategories(categories, structure);

    // Merge the structure offer with the personal offers of all members of the structure
    structure.personalOffers.forEach((personalOffer) => {
      const personalOfferDisplay = this.setServiceCategories(categories, personalOffer);

      // use lodash _.union fonction to concat array without duplicates
      structure.categoriesDisplay.onlineProcedures = _.union(
        structure.categoriesDisplay.onlineProcedures,
        personalOfferDisplay.categoriesDisplay.onlineProcedures,
      );
      structure.categoriesDisplay.baseSkills = _.union(
        structure.categoriesDisplay.baseSkills,
        personalOfferDisplay.categoriesDisplay.baseSkills,
      );
      structure.categoriesDisplay.advancedSkills = _.union(
        structure.categoriesDisplay.advancedSkills,
        personalOfferDisplay.categoriesDisplay.advancedSkills,
      );
    });
  }

  /**
   * Check if trainings are selected in order to ask for pricing or reset field "autres"
   */
  public isOtherAccompanimentSelected(structureForm: UntypedFormGroup): boolean {
    return Boolean(structureForm.value.categories?.onlineProcedures.find((el) => el === 'autres'));
  }

  public setFocusOnElement(elementRef: ElementRef<any>, cssSelector: string): void {
    const filterButton = elementRef.nativeElement.querySelector(cssSelector);
    if (filterButton) {
      const focusedElement = filterButton as HTMLElement;
      focusedElement.focus();
    }
  }

  /*
   * Sanitize value to null if empty
   */
  public nullifyEmpty(value: any): any {
    return value === undefined || value === null || value === '' ? null : value;
  }

  /*
   * Returns true if a string is empty or untouched
   */
  public isNullOrEmpty(value: string): boolean {
    return this.nullifyEmpty(value) === null;
  }

  /**
   * Convert and download to csv
   */
  public convertAndDownloadCsv(data: any[], filename: string): void {
    const csv = this.convertToCSV(data);
    this.downloadCSV(csv, filename);
  }

  private convertToCSV(data: any[]): string {
    const headers = Object.keys(data[0]).join(';'); // CSV headers

    const rows = data.map((row) => {
      return Object.values(row)
        .map((value) => {
          // Convert the value to a string
          let strValue = String(value);

          // If the value contains double quotes, replace them with two double quotes
          if (strValue.includes('"')) {
            strValue = strValue.replace(/"/g, '""');
          }

          // If the value contains commas, newlines, or apostrophes, enclose it in double quotes
          if (strValue.includes(';') || strValue.includes('\n') || strValue.includes("'")) {
            strValue = `"${strValue}"`;
          }

          return strValue;
        })
        .join(';'); // Join fields with commas
    });

    return [headers, ...rows].join('\n'); // Return the headers and data rows
  }

  private downloadCSV(csv: string, filename: string): void {
    const BOM = '\uFEFF'; // Special character at the beginning of the file for Excel encoding
    csv = BOM + csv;
    csv = csv.replace(/\n/g, '\r\n'); // Ensure proper line breaks in CSV
    const blob = new Blob([csv], { type: 'text/csv;charset=utf-8' });
    const url = window.URL.createObjectURL(blob);
    const a = document.createElement('a');
    a.setAttribute('href', url);
    a.setAttribute('download', filename);
    a.click();
    window.URL.revokeObjectURL(url); // Clean up URL
  }
}
